require 'ptstore'

require 'catalog'

def unescape(string)
  string.tr('+', ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
    [$1.delete('%')].pack('H*')
  end
end

def spEscape(string)
  string.gsub(/([^ a-zA-Z0-9_.-]+)/n) do
    $1.unpack('H2' * $1.size).join('')
  end.tr(' ', '+')
end


def query
  r = {}
  ENV['QUERY_STRING'].split('&').each do |kv|
    kv =~ /^([^=]+)=(.*)/
    k = $1
    v = $2
    r[k] = unescape(v)
    r[k+"_"] = v
  end
  r
end

def safe(msg)
  return nil if !msg
  msg.gsub(/&/, '&amp;').gsub(/</, '&lt;').gsub(/>/, '&gt;').gsub(/"/, '&quot;')
end

def url(msg)
  msg.gsub(/((ftp|https|gopher|http|telnet):\/\/[^\s\n]+)/,
           '<a href="\1">\1</a>')
end

def get_udb
  udb = PTStore.new('users.db')
  unless File.exist?('users.db')
    udb.transaction do
    end
  end
  udb
end

def get_users(udb)
  u = udb.roots.grep(/^[^_][^_]/).sort.join(', ')
  c(MEMBERS, u)
end

def get_topic(udb)
  c(TOPIC, udb['__topic'])
end

def get_hall
  return '' if !File.exists?('cache.txt')
  ret = ''
  File.open('cache.txt', 'r') do |i|
    i.flock(File::LOCK_SH)
    ret = i.read
    i.flock(File::LOCK_UN)
  end
  ret
end

def login(udb, u, img)
  addr = ENV['REMOTE_ADDR']
  if udb.root?(u)
    if udb[u][:addr] != addr
      raise "#{u}ϴ˻ѤƤ̾Ǥ"
    end
    udb[u][:last] = Time.now
  else
    udb[u] = {}
    udb[u][:img] = img
    udb[u][:addr] = addr
    udb[u][:last] = Time.now

	login_msg = LOGIN_MSGS[rand(LOGIN_MSGS.size)]
    append_sys_log_nolock(udb, c(login_msg, u))

    require 'plugin'
    plugin = PluginMgr.new(udb, u, img)
    plugin.apply_sys_plugin(:login)
  end
end

def sweep_timeout(udb)
  update = false
  now = Time.now
  udb.roots.each do |u|
    next if u =~ /^__/
    t = now - udb[u][:last]
    if !udb[u].key?(:pid) && ((udb[u].key?(:ajax) && t > 60) || t > 300)
      append_sys_log_nolock(udb, c(TIMEOUT_MSG, u))
      udb.delete(u)
      update = true

      require 'plugin'
      plugin = PluginMgr.new(udb, u, '')
      plugin.apply_sys_plugin(:timeout)
    end
  end
  update
end

def append_log(udb, msg)
  day = Time.now.day
  if !udb.root?('__logdump')
    udb['__logdump'] = day
  elsif udb['__logdump'] != day
    udb['__logdump'] = day

    require 'fileutils'
    require 'catalog'

    dat = File.open('chat.txt', 'r') do |ifile|
      ifile.readlines
    end.select {|l| l !~ /^<!-- OLD -->/}

    File.open('chat.txt', 'w') do |ofile|
      dat.reverse[0...20].reverse.each do |l|
        ofile.print "<!-- OLD -->#{l}"
      end
    end

	ds = dat.join

	ds.gsub!(/src="/, 'src="../../')
	foot = FOOT.gsub(/href="/, 'href="../../')
	ds.gsub!(/..\/..\/http/, 'http')
	ds.gsub!(/class="whisper"/, 'class="wopen"')
	ds.gsub!(/class="think"/, 'class="topen"')
	foot.gsub!(/..\/..\/http/, 'http')

    now = Time.now - 60*60*24
    year = now.year
    mon = now.mon
    day = now.day
    logdir = now.strftime("logs/%Y%m")
    FileUtils.mkdir_p(logdir)
    logfile = now.strftime"#{logdir}/%d.html"
    File.open(logfile, 'w') do |logf|
      logf.puts %Q(#{HEAD.sub(/teatime.css/, '../../teatime.css')}
</head>
<body>
<h1>#{TITLE} - #{now.strftime('%Y/%m/%d')}</h1>
<div id="hall"><div class="log"><table class="logList">
#{ds}</table></div></div>
#{foot}
)
    end
  end

  File.open('chat.txt', 'a') do |ofile|
    ofile.flock(File::LOCK_EX)
    ofile.puts(msg)
    ofile.flock(File::LOCK_UN)
  end
end

def append_stream(udb, user, msg)
  udb.roots.each do |u|
    next if u == user || u =~ /^__/
    if udb[u].key?(:stream)
      udb[u][:stream].push(msg)
    end
  end
end

def append_act_log(udb, msg)
  udb.transaction do
    append_act_log_nolock(udb, msg)
  end
end

def append_act_log_nolock(udb, msg)
  msg = %Q(<tr class="c"><td> </td><td><em class="act">[#{msg}]</em> <small>(#{Time.now.strftime('%H:%M:%S')})</small></td></tr>)
  append_log(udb, msg)
  append_stream(udb, nil, msg)
end

def append_sys_log(udb, msg)
  udb.transaction do
    append_sys_log_nolock(udb, msg)
  end
end

def append_sys_log_nolock(udb, msg)
  msg = %Q(<tr class="c"><td> </td><td><strong class="sys">[#{msg}]</strong> <small>(#{Time.now.strftime('%H:%M:%S')})</small></td></tr>)
  append_log(udb, msg)
  append_stream(udb, nil, msg)
end

def append_whisper_log(udb, user, img, to, msg)
  udb.transaction do
    append_whisper_log_nolock(udb, user, img, to, msg)
  end
end

def append_whisper_log_nolock(udb, user, img, to, msg)
  if udb.roots.grep(/^[^_][^_]/).sort.index(to)
	  msg = %Q(<tr class="#{spEscape(user)}"><td class="#{spEscape(to)}" nowrap><span class="whisper"><img src="#{img}.png"> <strong>#{user}</strong>:</span></td><td class="#{spEscape(to)}" width="100%"><span class="whisper">[&gt;#{to}] #{msg} <small>(#{Time.now.strftime('%H:%M:%S')})</small></span></td></tr>)
  else
	  msg = %Q(<tr class="#{spEscape(user)}"><td class="#{spEscape(to)}" nowrap><span class="think"><img src="#{img}.png"> <strong>#{user}</strong>:</span></td><td class="#{spEscape(to)}" width="100%"><span class="think">[Ȥ(#{to})] #{msg} <small>(#{Time.now.strftime('%H:%M:%S')})</small></span></td></tr>)
  end
  append_log(udb, msg)
  append_stream(udb, user, msg)
end

def append_chat_log(udb, user, img, msg)
  udb.transaction do
    append_chat_log_nolock(udb, user, img, msg)
  end
end

def append_chat_log_nolock(udb, user, img, msg)
  msg = %Q(<tr class="c"><td nowrap><img src="#{img}.png"> <strong>#{user}</strong>:</td><td width="100%">#{msg} <small>(#{Time.now.strftime('%H:%M:%S')})</small></td></tr>)
  append_log(udb, msg)
  append_stream(udb, user, msg)
end

