#!/usr/bin/env ruby

require 'util'
require 'nkf'
require 'mkdat'
require 'catalog'
require 'plugin'

cgi = query()
@udb = get_udb()
@user = safe(cgi['u'])
@img = safe(cgi['i'])
@plugin = PluginMgr.new(@udb, @user, @img)

@udb.transaction do
  login(@udb, @user, @img)
  sweep_timeout(@udb)
end

def submit_logout(cgi)
  logout_msg = LOGOUT_MSGS[rand(LOGOUT_MSGS.size)]
  @udb.transaction do
    append_sys_log_nolock(@udb, c(logout_msg, @user))
    @udb.delete(@user)
    mkdat_udb(@udb, @user)
  end

  @plugin.apply_sys_plugin(:logout)

  print %Q(Status Code: 302 Moved Temporary
Location: index.cgi

)
end

def submit_talk(cgi, is_utf8 = false)
  user = safe(@user)
  img = safe(cgi['i'])
  msg = cgi['msg']
  if msg == ''
    # reloading
    mkdat(@user)
    return
  end
  msg = url(safe(msg))
  msg = NKF.nkf('-We', msg) if is_utf8
  msg = @plugin.apply_plugin(msg)
  return if msg == ''

  append_chat_log(@udb, @user, img, msg)

  mkdat(@user)
end

def submit_topic(cgi, is_utf8 = false)
  user = safe(@user)
  topic = url(safe(cgi['topic']))
  return if topic == ''
  topic = NKF.nkf('-We', topic) if is_utf8
  @udb.transaction do
    append_sys_log_nolock(@udb, c(TOPIC_MSG, @user, topic))
    @udb['__topic'] = %Q(#{topic} by #{user})
    mkdat_udb(@udb, @user)
  end
end

def submit_act(cgi, is_utf8 = false)
  user = safe(@user)
  act = url(safe(cgi['act']))
  act = NKF.nkf('-We', act) if is_utf8
  actSelect = safe(cgi['act_select'])
  return if act == '' && actSelect == '00'

  act = act + ac(actSelect) if actSelect != '00'

  append_act_log(@udb, "#{@user}ϡ#{act}")

  mkdat(@user)
end

def response_ajax
  print %Q(Content-Type: text/plain; charset=EUC-JP
Cache-Control: no-cache
Pragma: no-cache

)
  puts get_hall()
end

def response_normal(cgi)
  print %Q(Status Code: 302 Moved Temporary
Location: index.cgi?u=#{cgi['u_']}&i=#{cgi['i']}

)
end

is_ajax = false
begin
  if cgi.key?('logoutSubmit')
    submit_logout(cgi)

  elsif cgi.key?('talkSubmit')
    submit_talk(cgi)
    response_normal(cgi)

  elsif cgi.key?('topicSubmit')
    submit_topic(cgi)
    response_normal(cgi)

  elsif cgi.key?('actSubmit')
    submit_act(cgi)
    response_normal(cgi)

  elsif cgi.key?('msg')
    is_ajax = true
    submit_talk(cgi, true)
    response_ajax

  elsif cgi.key?('topic')
    is_ajax = true
    submit_topic(cgi, true)
    response_ajax

  elsif cgi.key?('act')
    is_ajax = true
    submit_act(cgi, true)
    response_ajax

  else
    raise 'ʥꥯȤǤ'
  end
rescue
  if is_ajax
    print %Q(Content-Type: text/plain; charset=EUC-JP
Cache-Control: no-cache
Pragma: no-cache

<h1>ʤ󤫥顼</h1>
<p>#{$!}</p>
<pre>#{$!.backtrace.join("\n")}</pre>
<p>Ŭ˥ɤǤ⤷ƤߤƲ</p>
)
  else
    print "Content-Type: text/html; charset=EUC-JP\r\n\r\n"
    print HEAD
    print "<h1>ʤ󤫥顼</h1>"
    print "<p>#{$!}</p>"
    print %Q(<pre>#{$!.backtrace.join("\n")}</pre>)
    print "<p>֥饦뤢äƤ</p>"
  end
end

